#include <Wire.h>
#include <Rotary.h>
#include <U8g2lib.h>
#include <si5351.h>

// ------------------------------------------------------------------------------------------------------

Si5351 si5351;
Rotary r = Rotary(3, 2);
U8G2_SSD1306_128X64_NONAME_1_4W_HW_SPI u8g2(U8G2_R0, /* cs=*/ 8, /* dc=*/ 10, /* reset=*/ 9);

// ------------------------------------------------------------------------------------------------------
// ------------------------------------   PINES     -----------------------------------------------------
// ------------------------------------------------------------------------------------------------------

#define PinMenu 4

// ------------------------------------------------------------------------------------------------------
// ------------------------------------   definiciones del Usuario --------------------------------------
// ------------------------------------------------------------------------------------------------------

uint32_t F_MAX = 30000000UL;  // Límite superior de frecuencia
uint32_t F_MIN = 7000000UL;   // Límite inferior de frecuencia
uint32_t F_FI[3] = {10700000UL, 12000000UL, 4000000UL}; // 3 Frecuencias Intermedias. Si=0 no sale nada

int16_t Correccion = 500;     // Partes por billón de corrección del cristal. Ajustar el VCO con precisión
// Si IndiceOffset es 1= F + IF, si es 2= F - IF,  si es 3=IF - F, si es 4= OFFSET es 0
uint8_t IndiceOffset = 4;     // Indice al offset seleccionado para la frecuencia intermedia

/**************************************************************************
   Declaraciones
**************************************************************************/
extern const uint32_t StepSize[];
uint8_t  Indice_FI = 0;
uint8_t steps = 3;             // Almacena el puntero del step
long radix = StepSize[steps];  // Array con los valores de los steps

uint32_t vfo_t = F_MIN;
uint32_t xt_freq;              // Frecuencia del cristal

// ------------------------------------------------------------------------------------------------------

#include "utiles.h"            // Librería local de la funciones gráficas y de tiempo

/**************************************/
/* Rutina de servicio de interrupción */
/**************************************/
/* Esta es la interrupción del encoder.
   EncoderVal es +1 o -1 según haya subido o bajado. */

ISR(PCINT2_vect) {
  char result = r.process();   // Procesa el encoder (Librería encoder)
  if (result == DIR_CW) {
    EncoderVal ++;
    return;
  }
  if (result == DIR_CCW) {
    EncoderVal --;
    return;
  }
}

/*************************************************************************
  ---- ActualizaFsalida ----
  Esta función modifica la frecuencia de salida del si5351.
  Añade el offset de la frecuencia intermedia, según sea la seleccionada
**************************************************************************/
void ActualizaFsalida(void) {
  uint32_t tmp;
  tmp = vfo_t;

  // Se suma o resta la frecuencia intermedia seleccionada
  switch (IndiceOffset) {
    case 1: // se ha seleccionado + Frecuencia Intermedia
      tmp += F_FI[Indice_FI];
      break;
    case 2: // se ha seleccionado - Frecuencia Intermedia
      if (F_FI[Indice_FI] >= tmp)tmp = 50000UL; // se comprueba que la resta no es inferior a la mínima establecida
      tmp -= F_FI[Indice_FI];
      break;
    case 3: // se ha seleccionado Frecuencia Intermedia FI - la frecuencia del display
      if ( tmp >= F_FI[Indice_FI] )tmp = 50000UL; // se comprueba que la resta no es inferior a la mínima establecida
      else tmp = F_FI[Indice_FI] - tmp;
      break;
    case 4: // offset FI=0
      break;
  }
  si5351.set_freq((tmp * SI5351_FREQ_MULT), SI5351_PLL_FIXED, SI5351_CLK0); // establece la salida del CLK0

  // Establece la salida de CLK1 2ª FI
  if (F_FI[1] > 0) {
    si5351.output_enable(SI5351_CLK1, 1);
    si5351.set_freq((F_FI[1] * SI5351_FREQ_MULT), SI5351_PLL_FIXED, SI5351_CLK1);
    si5351.drive_strength(SI5351_CLK1, SI5351_DRIVE_2MA);
  }
  else si5351.output_enable(SI5351_CLK1, 0);

  // Establece la salida CLK2 3ª FI
  if (F_FI[2] > 0) {
    si5351.output_enable(SI5351_CLK2, 1);
    si5351.set_freq((F_FI[2] * SI5351_FREQ_MULT), SI5351_PLL_FIXED, SI5351_CLK2);
    si5351.drive_strength(SI5351_CLK2, SI5351_DRIVE_2MA);
  }
  else si5351.output_enable(SI5351_CLK2, 0);
}

/*************************************************************************
  AjustaFrecuencia
  Modifica la frecuencia principal cuando se cambia el encoder
**************************************************************************/

void AjustaFrecuencia() {
  int32_t tmp;
  radix = StepSize[steps];
  tmp = radix * EncoderVal;

  if (EncoderVal < 0 && abs(tmp) >= vfo_t) vfo_t = 50000UL;
  else   vfo_t += tmp;
  if (vfo_t > F_MAX) vfo_t = F_MAX;
  if (vfo_t < F_MIN) vfo_t = F_MIN;
  EncoderVal = 0;                 // resetea el valor del encoder para la siguiente lectura
  ActualizaFsalida();             // esta es la que cambia la frecuencia del si5351
}

/*****************************************************************************************
  Inicializa el circuito si5351
 *****************************************************************************************/

void Inicia_si5351() {
  si5351.set_correction(Correccion * 10);  // Ajusta el valor de corrección para el cristal de reloj
  si5351.init(SI5351_CRYSTAL_LOAD_10PF, 0); // Si se usa un cristal de 27Mhz poner 27000000 en lugar de 0
  // 0 is the default crystal frequency of 25Mhz.
  si5351.set_pll(SI5351_PLL_FIXED, SI5351_PLLA);
  ActualizaFsalida;
  si5351.drive_strength(SI5351_CLK0, SI5351_DRIVE_2MA);
}

//---------------------------------------------------------------------------------------------

void setup(void) {

  pinMode(2, INPUT_PULLUP);           // configura como entrada pull-up el pin del encoder
  pinMode(3, INPUT_PULLUP);           // configura como entrada pull-up el pin del  encoder

  pinMode(PinMenu, INPUT_PULLUP);     // configura como entrada pull-up el pin del pulsador

  // habilita interrupciones de los pines 2 y 3 del encoder
  PCICR |= (1 << PCIE2);              // habilita interrupcón por cambios en los pins del encoder
  PCMSK2 |= (1 << PCINT18) | (1 << PCINT19);
  sei();

  u8g2.begin();                       // configuramos librería gráfica

  Inicia_si5351();                    // Incializa el SI5351
  PrintMenu ();                       // .... dibuja pantalla inicial
  ActualizaFsalida();                 // actualiza la salida del Si5351
}

// ------------------------------------------------------------------------------------------------------

void loop(void) {

  if (Tiempo10mseg()) {              // cada 10ms lee el encoder
    leerswitch();
    if (EncoderVal != 0) {           // si encoderVal no es cero, es que se ha movido
      AjustaFrecuencia();            // ajusta frecuencia (subido o bajado)
      PrintMenu();                   // imprime la pantalla
      ActualizaFsalida();            // actualiza la salida del Si5351
    }
    switch (IsButtonPressed()) {
      case 0:                        // no está pulsado
        break;
      case 1:                        // si pulsación corta...
        steps ++;                    // incrementa el step
        if (steps > 6)steps = 1;     // si ha llegado a 6 (1MHz) vuelve a 1 (10Hz)
        PrintMenu ();                // imprime la pantalla     
        break;                    
      case 2:                        // Pulsación larga. Seleccionamos la primera línea del menú, del menú normal
        break;
    }
// ------------------------------------------------------------------------------------------------------
  }
// ---- Fin flag 10 mseg ------------------------------------------------------------------------------
}


